
; ==========================================================================
; (c) Ilya Mamontov, 2008
; Thermometer
; ==========================================================================
; timer8_1int.asm file contains user code


include "m8c.inc"       ; part specific constants and macros
include "PSoCAPI.inc"   ; PSoC API definitions for all User Modules
include "flashblock.inc"
include "math.inc"

;------------------------------------------------------------------------------
K20:	equ 20	;temperature1
K99:	equ 100	;temperature2
AVG:	equ 8	;averaging factor

area mydata(ROM,ABS)
FLASH_block:	EQU	0x1000	;absolute address
org FLASH_block
.LITERAL
FLASH_block1:	;64 bytes of FLASH for parameters
DWL 5350		;K1_0 2710
 DB 0,0,0
DWL 5350		;K2_0 2677
 DB 0,0,0
DWL 5350		;K3_0 2753
 DB 0,0,0
DWL 3800		;Kr_0
 DB 0,0,0

DWL 7000		;K1_1
 DB 0,0,0
DWL 7000		;K2_1
 DB 0,0,0
DWL 7000		;K3_1
 DB 0,0,0
DWL 3800		;Kr_1
 DB 0,0,0
.ENDLITERAL

;------------------------------------------------------------------------------
area bss(RAM,REL)
;block of parameters to store
	RAM_block:
K1_0:		BLK	5	;coefficient
K2_0:		BLK	5	;coefficient
K3_0:		BLK	5	;coefficient
Kr_0:		BLK	5	;coefficient
K1_1:		BLK	5	;coefficient
K2_1:		BLK	5	;coefficient
K3_1:		BLK	5	;coefficient
Kr_1:		BLK	5	;coefficient
	RAM_block_end:

N1::		BLK	5	;Result
N2::		BLK	5	;Result
N3::		BLK	5	;Result
Nr::		BLK	5	;Result
No::		BLK	5	;Result

NN1:		BLK	5	;average
NN2:		BLK	5	;average
NN3:		BLK	5	;average
NNr:		BLK	5	;average

cnt:		BLK	1	;counter for averaging
cnt2:		BLK	1	;temp
var0:		BLK	5	;temp
var1:		BLK	5	;temp
var2:		BLK	5	;temp
var3:		BLK	5	;temp

temp8:		BLK	8	;FLASH_WRITE_STRUCT structure for bFlashWriteBlock procedure
;------------------------------------------------------------------------------
;  DESCRIPTION:	Calculates result and converts it into 7-seg string (two digits).
;				Checks the ranges.
;  ARGUMENTS:	@0 - number of channel (1, 2 or 3)
;  RETURNS:		none
;------------------------------------------------------------------------------
macro toSEG @0
	;normalize Kn_0 * Nr/Kr_0
	math_mul5x3 var0,K@0_0,NNr
	math_div5 var0,Kr_0	
	;normalize Kn_1 * Nr/Kr_1
	math_mul5x3 var1,K@0_1,NNr
	math_div5 var1,Kr_1
	;t=(99-20)/(U1-U0) * (U-U0) + 20
	mov [math_flag],0			;clear condition flag
	math_sub5 var1,var1,var0	;var1 = (U1-U0)
	math_abs5 var1				;|var1|
	math_sub5 var0,NN@0,var0	;var0 = (U-U0)
	math_abs5 var0				;|var0|
	math_load5 var2,0,(K99-K20)	;var2 = 99 - 20
	math_mul5x3 var3,var0,var2	;var3 = var0 * var2
	math_div5 var3,var1			;var3 = var3 / var1
	math_abs_neg5 var3			;condition |var3|
	math_load5 var1,0,K20		;var1 = 20
	math_add5 var3,var3,var1	;var3 = var2 + var1


	;check the range
	math_load5 var1,0,119		;max value
	math_sub5  var0,var1,var3	;check
	tst [var0+4],80h
	jnz .toSEG_max
	math_load5 var1,0,19		;min negative value
	math_add5  var0,var1,var3	;check
	tst [var0+4],80h
	jnz .toSEG_min

	;display the result
	math_abs5 var3				;|var3|
	mov A,[var3]
	mov X,(an+@0+@0-2)
	call to99
	;check the sign
	tst [math_flag],80h
	jz .toSEG_pos
		or [an+@0+@0-2],40h		;set minus
	.toSEG_pos:
	jmp .toSEG_end

.toSEG_max:	;case of "out of range", positive
	mov [an+@0+@0-2],23h	;"n"
	mov [an+@0+@0-1],23h	;"n"
	jmp .toSEG_end
.toSEG_min:	;case of "out of range", negative
	mov [an+@0+@0-2],40h	;"-"
	mov [an+@0+@0-1],40h	;"-"
.toSEG_end:



endm

;------------------------------------------------------------------------------
area text(ROM,REL)
_main::
	;initialize variables
	call FLASH_restore	;load initial values from FLASH

	;prepare for 7-seg display multiplexing 
    mov [aa],0         ;load initial offset for "jmp" instruction
    mov [ai],an        ;load address of pattern buffer
	; load initial image to pattern buffer
	; nums:     0   1   2   3   4   5   6   7   8   9
	; images: 3Fh,06h,5Bh,4Fh,66h,6Dh,7Dh,07h,7Fh,6Fh
    mov [an+0],FFh
    mov [an+1],FFh
    mov [an+2],FFh
    mov [an+3],FFh
    mov [an+4],FFh
    mov [an+5],FFh

	mov A,3
    call RefMux_1_Start
	mov A,3
    call PGA_1_Start
	mov A,3
    call PGA_2_Start
	or reg[PGA_1_GAIN_CR3],2	;differential amp connection
	or reg[PGA_2_GAIN_CR3],2	;differential amp connection

	mov A,3
    call ADCINC_1_Start
	mov A,5
    call ADCINC_1_GetSamples
    call Timer8_1_Start
	call Timer8_1_EnableInt
	M8C_EnableGInt
	;Dinamic indication is implemented in Timer8_1INT.asm

loop: ;---------------------------------------------------------------------------------------------

	;clear accumulators for averaging 
	math_load5 NN1,0,0
	math_load5 NN2,0,0
	math_load5 NN3,0,0
	math_load5 NNr,0,0
	mov [cnt],0

wait: ;wait for ADC results (N1, N2, N3, Nr)   
	call ADCINC_1_fIsDataAvailable	;poll flag   
	jz wait   
	call ADCINC_1_wClearFlagGetData	;reset flag and retrieve data X:A

	;check keys
	tst [key_cnt],10h	;key pressed?
	jnz loop			;start new cycle

	;accumulate for averaging
	math_add5 NN1,NN1,N1
	math_add5 NN2,NN2,N2
	math_add5 NN3,NN3,N3
	math_add5 NNr,NNr,Nr

cmp [cnt],(AVG-1)
jnz .m2
xor [an+1],80h ;i'm live!
xor [an+3],80h ;i'm live!
xor [an+5],80h ;i'm live!
.m2:


	inc [cnt]
	cmp [cnt],AVG	;averaging factor
	jc wait			;accumulating loop

	;average results
	math_load5 var1,0,AVG
	math_div5 NN1,var1
	math_div5 NN2,var1
	math_div5 NN3,var1
	math_div5 NNr,var1

	;check keys and save new parameters
	tst [key_cnt],80h	;key was pressed?
	jz .m3				;no
	mov [key_cnt],0	;clear flag
		;save new coeff values
		call FLASH_restore
		math_mov5 K1_0,NN1
		math_mov5 K2_0,NN2
		math_mov5 K3_0,NN3
		math_mov5 Kr_0,NNr
		call FLASH_save
	.m3:
	tst [key_cnt],40h	;key was pressed?
	jz .m4				;no
	mov [key_cnt],0	;clear flag
		;save new coeff values
		call FLASH_restore
		math_mov5 K1_1,NN1
		math_mov5 K2_1,NN2
		math_mov5 K3_1,NN3
		math_mov5 Kr_1,NNr
		call FLASH_save
	.m4:


IF 1 ;1 - normal mode,  0 - debug mode
	m1: toSEG 1	;calculate and display result
	m2: toSEG 2	;calculate and display result
	m3: toSEG 3	;calculate and display result
	;turn on decimal points
	or [an+1],80h
	or [an+3],80h
	or [an+5],80h

ELSE ;show debug info (N1,N2,N3,Nr)
				cmp [an+0],5Ch
				jnz .n1
					mov X,NN1
					call to999
					mov [an+0],10h	;10h 18h 1Ch 5Ch 
					jmp .n4
				.n1:
				cmp [an+0],10h
				jnz .n2
					mov X,NN2
					call to999
					mov [an+0],18h	;10h 18h 1Ch 5Ch 
					jmp .n4
				.n2:
				cmp [an+0],18h
				jnz .n3
					mov X,NN3
					call to999
					mov [an+0],1Ch	;10h 18h 1Ch 5Ch 
					jmp .n4
				.n3:
					mov X,NNr
					call to999
					mov [an+0],5Ch	;10h 18h 1Ch 5Ch 
					jmp .n4
				.n4:
ENDIF ;show debug info (NN1,NN2,NN3,NNr)

	jmp loop  
    ret


    
;----------------------------------------------------------------------------------
; FUNCTION NAME: FLASH_restore
; DESCRIPTION:	Restores parameters from FLASH
; ARGUMENTS:
;----------------------------------------------------------------------------------
FLASH_restore::
	;FlashReadBlock procedure is not used (any FLASH address is allowed)
	mov X,<FLASH_block						;LSB of FLASH address
	mov A,>FLASH_block						;MSB of FLASH address
	mov [temp8+0],RAM_block					;LSB of RAM address
	mov [temp8+2],(RAM_block_end-RAM_block)	;bytes to read
	.FLASH_restore_loop:
	push A
	romx
	mvi [temp8+0],A
	pop A
	inc X
	adc A,0
	dec [temp8+2]
	jnz .FLASH_restore_loop

	;------- check critical variables
	;call correct_parameters
	;------- restore PSOC registers
	ret

;----------------------------------------------------------------------------------
; FUNCTION NAME: FLASH_save
; DESCRIPTION:	Saves parameters from FLASH
; ARGUMENTS:
;----------------------------------------------------------------------------------
FLASH_save::
	;------- 1-st block (64 bytes)
	mov X,temp8								;LSB of FLASH_WRITE_STRUCT address
	mov A,0									;MSB of FLASH_WRITE_STRUCT address
	mov [X+0],0								;FLASH block Id MSB=0
	mov [X+1],FLASH_block>>6				;FLASH block Id (=address/64)
	mov [X+2],0								;MSB of RAM address
	mov [X+3],RAM_block						;LSB of RAM address
	mov [X+4],25							;temperature value
	inc X					;correct FLASH_WRITE_STRUCT address for 27xxx
	lcall bFlashWriteBlock	;stack depth is 14 bytes!!
	ret
;------------------------------------------------------------------------------ 
	