;======================================================
;
; math.inc v1.4     (c)2004 Ilya Mamontov 
;
; Macro definitions for 5-byte (40-bit) mathematics
;
;======================================================
macro math_add5 ;@0, @1, @2
;usage: math_add5 var1,var2,var3
;result:          var1=var2+var3
;CF=1 if unsigned overflow
	mov [math_aa1],@1
	mov		   X,  @2
	mov [math_aa2],@0
	lcall math__add5
endm
;-----------------------------------------------
macro math_sub5 ;@0, @1, @2
;usage: math_sub5 var1,var2,var3
;result:          var1=var2-var3
;CF=1 if unsigned underflow
	mov [math_aa1],@1
	mov		   X,  @2
	mov [math_aa2],@0
	lcall math__sub5
endm
;-----------------------------------------------
macro math_mul5x3 ;@0, @1, @2
;usage: math_mul5x3 var1,var2,var3 (var3 should be a 3-byte integer)
;result:            var1=var2*var3
;(!)var1 and var2 must be different!(e.g.: string "math_mul5x3 a,a,b" is wrong!)
	mov [math_aa1],@1
	mov [math_aa2],@2
	mov		   X,  @0
	lcall math__mul5x3
endm
;-----------------------------------------------
macro math_div5  ;@0, @1
;usage: math_div5 var1,var2
;result:     var1=var1/var2, remainder is in 5-byte "math_remainder"
;(!)var1,var2 should be unsigned
;(!)no check for division by 0
	mov		   X,  @0
	mov [math_aa1],@1
	lcall math__div5
endm
;-----------------------------------------------
macro math_neg5 ;@0
;usage: math_neg5 var1
;result:    var1=-var1
	mov  X,@0
	lcall math__neg5
endm
;-----------------------------------------------
macro math_abs5 ;@0
;usage: math_abs5 var1
;result: var1=abs(var1),  bit 7 of "math_flag" is inverted if operand is negative (bits 0...6 are destroyed)
	mov  X,@0
	lcall math__abs5
endm
;-----------------------------------------------
macro math_abs_neg5 ;@0
;usage: math_abs_neg5 var1
;result:        var1=-var1 if bit 7 of "math_flag" is 1
	tst [math_flag],0x80
	jz math_abs_neg5_positive
	xor [math_flag],0x80
	mov  X,@0
	lcall math__neg5
math_abs_neg5_positive:
endm
;-----------------------------------------------
macro math_load5 ;@0, @1, @2
;usage: math_load5 var1,const1,const2
;result:           var1=const1_const2 (1 MS Byte and 4 LS Bytes)
	mov [@0+0],(@2 & FFh)
	mov [@0+1],(@2>>8  & FFh)
	mov [@0+2],(@2>>16 & FFh)
	mov [@0+3],(@2>>24 & FFh)
	mov [@0+4],(@1 & FFh)
endm
;-----------------------------------------------
macro math_mov5 ;@0, @1
;usage: math_mov5 var1,var2
;result:          var1 <= var2
	mov [@0+0],[@1+0]
	mov [@0+1],[@1+1]
	mov [@0+2],[@1+2]
	mov [@0+3],[@1+3]
	mov [@0+4],[@1+4]
endm
;-----------------------------------------------
