
;;*****************************************************************************
;;  FILENAME: to999.asm
;;  VERSION: 1.3b
;;
;;  DESCRIPTION: MultiByte Integer-to-7-seg String Conversion Routines:
;;               to99
;;               to999
;;
;;  NOTE:  These functions use internal variables which are static allocated.
;;         Do not call these functions from different threads! 
;;
;;  Copyright (c) Ilya Mamontov 2007. All Rights Reserved.
;;*****************************************************************************

include "m8c.inc"


area bss(RAM)
binary:		BLK	2	;work variable
bitCount:	BLK	1
byteCount:	BLK	1
bcd:		BLK	3	;BCD value


;------------------------------------------
area text(ROM,REL)

;-----------------------------------------------------------------------------
;  DESCRIPTION:	Converts an unsigned byte 0...119 into 7-seg string.
;				String is right-justified, length-fixed (2)
;  ARGUMENTS:	A - Byte
;				X - start position in buffer
;  RETURNS:		none
;  Example of usage:
;			mov X,(an+2)
;			mov A,[cnt2]
;			call to99
;-----------------------------------------------------------------------------
to99::
;------------------- to BCD ---------------------
	push X				;save pointer
	mov [binary+0],A	;load byte from A
	mov [binary+1],0

	mov [bitCount],8	;8 bit total
	mov [bcd+0],00h
	mov [bcd+1],00h

k0:	rlc [binary+0]
	rlc [bcd+0]
	rlc [bcd+1]
	dec [bitCount]
	jz k1				;if BCD Conversion is over

	mov [byteCount],2	;process 4 decimal digits
	mov X,bcd

b1: mov A,[X]
	add [X],03h
	tst [X],08h
	jnz b2
	mov [X],A
b2:	mov A,[X]
	add [X],30h
	tst [X],80h
	jnz b3
	mov [X],A
b3:	inc X
	dec [byteCount]
	jnz b1
	jmp k0

;-------------------- to string -----------------
k1:
	pop X				;restore pointer
	;-- 2nd digit
	mov A,[bcd+0]
	asr A
	asr A
	asr A
	asr A
	and A,0Fh
	jnz k_7			;
	mov a,0Ah		;remove "0" by " "
k_7:index table
	mov [X+0],A
	;-- 1st digit	
	mov A,[bcd+0]
	and A,0Fh
	index table
	mov [X+1],A

	;-- 3th "virtual" digit
	tst [bcd+1],0Fh		;>100?
	jz k_9
	cmp [X+0],00h		;"0"?
	jnz k_8
	mov [X+0],39h		;"C"
k_8:or  [X+0],30h		;"1"

k_9:
	ret


;-----------------------------------------------------------------------------
;  DESCRIPTION:	Converts a 2-byte unsigned Integer into 7-seg string.
;				String is right-justified, length-fixed (5)
;  ARGUMENTS:	X - address of Integer
;  RETURNS:		none
;-----------------------------------------------------------------------------
to999::
;------------------- to BCD ---------------------
	mov A,[X+0]			;load 2 bytes of "binary"
	mov [binary+0],A
	mov A,[X+1]
	mov [binary+1],A

	mov [bitCount],16	;16 bit total
	mov [bcd+0],00h
	mov [bcd+1],00h
	mov [bcd+2],00h		;delete this line if conversion is "single byte-to-bcd"!

m0:	rlc [binary+0]
	rlc [binary+1]		;delete this line if conversion is "single byte-to-bcd"!
	rlc [bcd+0]
	rlc [bcd+1]
	rlc [bcd+2]			;delete this line if conversion is "single byte-to-bcd"!
	dec [bitCount]
	jz m1				;if BCD Conversion is over

	mov [byteCount],3	;process 6 decimal digits; =2 (i.e. 4 dec dig) if conversion is "1 byte to bcd"
	mov X,bcd

c1: mov A,[X]
	add [X],03h
	tst [X],08h
	jnz c2
	mov [X],A
c2:	mov A,[X]
	add [X],30h
	tst [X],80h
	jnz c3
	mov [X],A
c3:	inc X
	dec [byteCount]
	jnz c1
	jmp m0

;-------------------- to string -----------------
m1:
	;-- 5th digit
	mov A,[bcd+2]
	and A,0Fh
	jnz m_4			;


	mov a,0Ah		;remove "0" by " "
m_4:index table
	mov [an+1],A
	;-- 4th digit
	mov A,[bcd+1]
	asr A
	asr A
	asr A
	asr A
	and A,0Fh
	jnz m_5			;
	cmp [an+1],0	;
	jnz m_5			;
	mov a,0Ah		;remove "0" by " "
m_5:index table
	mov [an+2],A
	;-- 3th digit
	mov A,[bcd+1]
	and A,0Fh
	jnz m_6			;
	cmp [an+2],0	;
	jnz m_6			;
	mov a,0Ah		;remove "0" by " "
m_6:index table
	mov [an+3],A
	;-- 2nd digit
	mov A,[bcd+0]
	asr A
	asr A
	asr A
	asr A
	and A,0Fh
	jnz m_7			;
	cmp [an+3],0	;
	jnz m_7			;
	mov a,0Ah		;remove "0" by " "
m_7:index table
	mov [an+4],A
	;-- 1st digit	
	mov A,[bcd+0]
	and A,0Fh
	index table
	mov [an+5],A
	ret
table: ;0   1   2   3   4   5   6   7   8   9   0A(None)
    DB 3Fh,06h,5Bh,4Fh,66h,6Dh,7Dh,07h,7Fh,6Fh, 00h	



;-----------------------------------------------------------------------------
;  DESCRIPTION:	Converts string into 2-byte Integer.
;
;  ARGUMENTS:	X - pointer to null terminated string
;  RETURNS:		A:X - MSB and LSB of Integer
;
;-----------------------------------------------------------------------------
to999_stoi::
	mov [binary+0],0	;reset result
	mov [binary+1],0
ms1:
	mov A,[X]			;check for Null character
	jz ms99				;return result
	;---multiply by 10
	asl [binary+0]
	rlc [binary+1]
	 mov          A,[binary+0]
	 mov [binary+2],[binary+1]
	asl [binary+0]
	rlc [binary+1]
	asl [binary+0]
	rlc [binary+1]
	 add [binary+0],A
	 mov A,[binary+2]
	 adc [binary+1],A
	;---add digit
	mov A,[X]			;load next digit
	inc X				;and increment pointer
	and A,0xF			;mask
	add [binary+0],A
	adc [binary+1],0
	jmp ms1
ms99:
	mov X,[binary+0]	;return LSB of integer
	mov A,[binary+1]	;return MSB of integer
	ret
	